!> \file flowlaw-0.3.f90
!!Subroutine qui calcul les parametres pour chaque loi de deformation
!<

!> SUBROUTINE: FLOWLAW
!! \author ...
!! \date 25 Novembre 1999
!!  
!!@note  subroutine qui calcul SA et S2A et BTT pour chaque loi de deformation (exposant variable)
!!
!! variables en entree : - iglen : precise sur quel exposant de la loi de deformation
!!                                 flowlaw doit travailler.
!!                       - glen : tableau contenant les exposants des lois
!!                       - BAT1 et BAT2
!!                       - TTRANS
!!                       - Q1 et Q2
!!                       - T
!!                       - RGAS
!!                       - TI2, TETAR et TETA : tableaux de travail calcule dans
!!                         la routine flow_general
!!                       - E
!!
!! variables en sortie : - SA et S2A
!!                       - BTT
!!
!! @note Used modules:
!! @note   - use module3D_phy
!! @note   - use module_choix
!>

SUBROUTINE FLOWLAW (iiglen)

  USE module3D_phy
  !      USE DEFORM_DECLAR
  USE module_choix

  implicit none
  INTEGER ::  iiglen   !< compteur pour la boucle flowlaw
  REAL :: a5 !< exposant de la loi de def +2 
  REAL :: a4 !< exposant de la loi de def +1 
  REAL :: TI1 !< utilise pour le calcul de btt a k=1
  REAL :: BAT !< prend la valeur de BAT1 ou BAT2 selon les cas
  REAL :: Q !< prend la valeur de Q1 ou Q2 selon les cas
  REAL :: AAT !< variable de travail =Q*TETAR(I,J,K)
  REAL :: SSEC !< variable de travail 
  REAL :: ZETAT !< variable de travail : profondeur ou T = TRANS(iiglen)
  REAL,dimension(NX,NY,NZ) :: SI1 !< tableau de calcul
  REAL,dimension(NX,NY,NZ) :: SI2 !< tableau de calcul
  real,dimension(nx,ny) :: tab_mx
  real,dimension(nx,ny) :: tab_my
  real,dimension(nx,ny) :: tab2D

  REAL,dimension(NZ) :: E          ! vertical coordinate in ice, scaled to H zeta
  REAL :: De= 1./(NZ-1)

  E(1)=0.
  E(NZ)=1.
  do K=1,NZ
     if ((K.ne.1).and.(K.ne.NZ)) E(K)=(K-1.)/(NZ-1.)
  end do

  ! exposant de la loi de deformation utilisee : glen(iiglen)
  ! l'exposant correspondant a iiglen est defini dans DEFORMATION_MOD
  a5 = glen(iiglen) + 2  
  a4 = glen(iiglen) + 1

  !    boucle pour btt a k=1
  TI1=273.15*273.15
  do J=2,NY-1
     do I=2,NX-1
        if (T(i,j,1).le.TTRANS(iiglen)) then
           BTT(I,J,1,iiglen)=BAT1(iiglen)*exp(Q1(iiglen)/RGAS/TI1*T(I,J,1))
        else
           BTT(I,J,1,iiglen)=BAT2(iiglen)*exp(Q2(iiglen)/RGAS/TI1*T(I,J,1))
        endif
     end do
  end do

  ! boucle pour tous les autres BTT
  do K=NZ-1,1,-1
     do J=2,NY-1
        do I=2,NX-1  
           if (H(i,j).gt.1.) then
              if ((TETA(I,J,K).le.TTRANS(iiglen)).and. &
                   (TETA(I,J,K+1).le.TTRANS(iiglen))) then
                 BAT=BAT1(iiglen)
                 Q=Q1(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SSEC/(AAT+a4)*(E(K)**(AAT+a4)-E(K+1)**(AAT+a4))
                 SI2(I,J,K)=((E(K)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a4))*E(K)+E(K+1)**(AAT+a5)) &
                      * SSEC/(AAT+a4) 
                 BTT(I,J,K+1,iiglen)=BAT*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
              else if ((TETA(I,J,K).ge.TTRANS(iiglen)).and. &
                   (TETA(I,J,K+1).ge.TTRANS(iiglen))) then
                 BAT=BAT2(iiglen)
                 Q=Q2(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SSEC/(AAT+a4)*(E(K)**(AAT+a4)-E(K+1)**(AAT+a4))
                 SI2(I,J,K)=((E(K)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a4))*E(K)+E(K+1)**(AAT+a5)) &
                      * SSEC/(AAT+a4) 
                 BTT(I,J,K+1,iiglen)=BAT*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))

                 ! cas ou la temperature de la maille est en partie au dessus et au dessous
                 ! de TTRANS(iiglen)
              else if ((TETA(I,J,K).LT.TTRANS(iiglen)).and. &
                   (TETA(I,J,K+1).GT.TTRANS(iiglen))) then
                 !         calcul de la profondeur de transition
                 if (abs(TETA(I,J,K)-TETA(I,J,K+1)).lt.1.e-5) then
                    ZETAT=(E(K)+E(K+1))/2.
                 else
                    ZETAT=E(K+1)-(TTRANS(iiglen)-TETA(I,J,K+1))/ &
                         (TETA(I,J,K)-TETA(I,J,K+1))*DE
                 endif

                 !         integration entre ZETA2 et ZETAT, loi bat2 
                 BAT=BAT2(iiglen)
                 Q=Q2(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SSEC/(AAT+a4)*(ZETAT**(AAT+a4)-E(K+1)**(AAT+a4))
                 SI2(I,J,K)=((ZETAT**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a4))*ZETAT+E(K+1)**(AAT+a5)) &
                      * SSEC/(AAT+a4)
                 BTT(I,J,K+1,iiglen)=BAT*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))

                 !         integration entre ZETAT et ZETA1, loi bat1 
                 BAT=BAT1(iiglen)
                 Q=Q1(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SI1(I,J,K)+ &
                      SSEC/(AAT+a4)*(E(K)**(AAT+a4)-ZETAT**(AAT+a4))
                 SI2(I,J,K)=SI2(I,J,K) &
                      +((E(K)**(AAT+a5))/(AAT+a5) &
                      -(ZETAT**(AAT+a5))/(AAT+a5) &
                      -(ZETAT**(AAT+a4))*E(K)+ZETAT**(AAT+a5)) &
                      * SSEC/(AAT+a4)


                 ! deuxieme cas ou la temperature de la maille est en partie au dessus et 
                 ! au dessous de TTRANS(iiglen)
              else if ((TETA(I,J,K).GT.TTRANS(iiglen)).and. &
                   (TETA(I,J,K+1).LT.TTRANS(iiglen))) then
                 !         calcul de la profondeur de transition
                 if (abs(TETA(I,J,K)-TETA(I,J,K+1)).lt.1.e-5) then
                    ZETAT=(E(K)+E(K+1))/2.
                 else
                    ZETAT=E(K+1)-(TTRANS(iiglen)-TETA(I,J,K+1))/ &
                         (TETA(I,J,K)-TETA(I,J,K+1))*DE
                 endif

                 !         integration entre ZETA2 et ZETAT, loi bat1 
                 BAT=BAT1(iiglen)
                 Q=Q1(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SSEC/(AAT+a4)*(ZETAT**(AAT+a4)-E(K+1)**(AAT+a4))
                 SI2(I,J,K)=((ZETAT**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a5))/(AAT+a5) &
                      -(E(K+1)**(AAT+a4))*ZETAT+E(K+1)**(AAT+a5)) &
                      * SSEC/(AAT+a4)
                 BTT(I,J,K+1,iiglen)=BAT*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))

                 !         integration entre ZETAT et ZETA1, loi bat2 
                 BAT=BAT2(iiglen)
                 Q=Q2(iiglen)
                 AAT=Q/RGAS/TI2(I,J,K)*(TETA(I,J,K+1)-TETA(I,J,K))/DE*E(K+1)
                 AAT=max(-1.8,AAT)
                 AAT=min(80.,AAT)
                 SSEC=BAT/(E(K+1)**AAT)*exp((Q*TETA(I,J,K+1))/(RGAS*TI2(I,J,K)))
                 SI1(I,J,K)=SI1(I,J,K)+ &
                      SSEC/(AAT+a4)*(E(K)**(AAT+a4)-ZETAT**(AAT+a4))
                 SI2(I,J,K)=SI2(I,J,K) &
                      +((E(K)**(AAT+a5))/(AAT+a5) &
                      -(ZETAT**(AAT+a5))/(AAT+a5) &
                      -(ZETAT**(AAT+a4))*E(K)+ZETAT**(AAT+a5)) &
                      * SSEC/(AAT+a4)
              endif
           endif
        end do
     end do
  end do

  !     integration de SA et S2A

  do J=1,NY
     do I=1,NX  
        SA(I,J,NZ,iiglen)=0.
        S2A(I,J,NZ,iiglen)=0.
        if (H(I,J).le.1.) BTT(I,J,NZ,iiglen)=BAT2(iiglen)
     end do
  end do

  do K=NZ-1,1,-1
     do J=1,NY
        do I=1,NX
           if (H(i,j).gt.1.) then
              SA(I,J,K,iiglen)=SA(I,J,K+1,iiglen)-SI1(I,J,K)
              S2A(I,J,K,iiglen)=S2A(I,J,K+1,iiglen)+SI2(I,J,K)+ &
                   SA(I,J,K+1,iiglen)*DE
           else
              SA(I,J,K,iiglen)=BAT2(iiglen)/a4*(1-E(K)**a4)
              S2A(I,J,K,iiglen)=BAT2(iiglen)/a4*(a4/a5-E(K)+E(K)**a5/a5)
              BTT(I,J,K,iiglen)=BAT2(iiglen)
           endif
        end do
     end do
  end do

  !     cas particulier des bords
  SA(:,1,:,iiglen)=SA(:,2,:,iiglen)
  S2A(:,1,:,iiglen)=S2A(:,2,:,iiglen)
  BTT(:,1,:,iiglen)=BTT(:,2,:,iiglen)
  SA(:,NY,:,iiglen)=SA(:,NY-1,:,iiglen)
  S2A(:,NY,:,iiglen)=S2A(:,NY-1,:,iiglen)
  BTT(:,NY,:,iiglen)=BTT(:,NY-1,:,iiglen)

  SA(1,:,:,iiglen)=SA(2,:,:,iiglen)
  S2A(1,:,:,iiglen)=S2A(2,:,:,iiglen)
  BTT(1,:,:,iiglen)=BTT(2,:,:,iiglen)
  SA(NX,:,:,iiglen)=SA(NX-1,:,:,iiglen)
  S2A(NX,:,:,iiglen)=S2A(NX-1,:,:,iiglen)
  BTT(NX,:,:,iiglen)=BTT(NX-1,:,:,iiglen)

  ! calcul des moyennes sur les mailles staggered

  do k=1,nz
     tab2D=SA(:,:,k,iiglen)

     call moy_mxmy(nx,ny,tab2D,tab_mx,tab_my)
     SA_mx(:,:,k,iglen)=tab_mx
     SA_my(:,:,k,iglen)=tab_my

     tab2D=S2A(:,:,k,iiglen)
     call moy_mxmy(nx,ny,tab2D,tab_mx,tab_my)
     S2A_mx(:,:,k,iglen)=tab_mx
     S2A_my(:,:,k,iglen)=tab_my
  end do

! attribution de debug_3D pour les sorties S2a
! loi 1 -> 190 dans description variable -> 90 dans debug_3D
debug_3d(:,:,90+iiglen-1) = S2a(:,:,1,iiglen)

END SUBROUTINE FLOWLAW
