!> \file icetempmod.f90
!! Implementation of functions and subroutines for temperature calculation 
!<

!> SUBROUTINE: Init_icetemp 
!! 
!! Initialise the temperature calculation
!! Used modules:
!! - Icetemp_declar
!>

Subroutine Init_icetemp(Num_rep_42_m)
  Use Icetemp_declar
  
  Implicit None

  !< Arguments
  Integer,                       Intent(In)::Num_rep_42_m !< Id Of Reponse
  ! Character(Len=80) :: Filin
  !________________________________________________________________
  ! La Temperature Du Point De Congelation De L'Eau De Mer
  ! Est Tiree De Jenkins (1991), Tb Est En Degres C
  ! _______________________________________________________________

  !  Nouvelle Formulation De Fracq
  !  Nfracq=15  ! Ancienne Formulation

  Nfracq=1

  Write(Num_rep_42_m,*) 'Calcul Des Temperatures'
  Write(Num_rep_42_m,*) 'Nfracq=',Nfracq

  Iq=1

  If (Iq.Eq.1) Write(Num_rep_42_m,*) 'Chaleur Demi Maille, Iq=',Iq
  If (Iq.Eq.2) Write(Num_rep_42_m,*) 'Chaleur Centree, Iq=',Iq
  If (Iq.Eq.3) Write(Num_rep_42_m,*) 'Chaleur Pente, Iq=',Iq
  If (Iq.Eq.4) Write(Num_rep_42_m,*) 'Chaleur U_taub, Iq=',Iq
  If (Iq.Eq.5) Write(Num_rep_42_m,*) 'Chaleur U_taub_stag, Iq=',Iq
  If (Iq.Eq.6) Write(Num_rep_42_m,*) 'Chaleur Q_sia_stag,Iq=',Iq
  If (Iq.Eq.7) Write(Num_rep_42_m,*) 'Chaleur Q_all_stag,Iq=',Iq
  Write(Num_rep_42,*)

  Ecart_phid=0.5
  Write(Num_rep_42_m,*) ' Si Base Froide, La Chaleur De Glissement Est En Exp(Delta T * Ecart_phid)'
  Write(Num_rep_42_m,*) 'Ecart_phid=',Ecart_phid
  Write(Num_rep_42_m,*) '---------------------------------------------------------------------------'

End Subroutine Init_icetemp

!> SUBROUTINE: Icetemp()
!! Calculate the temperature
!!
!! Used modules:
!!  -  use Icetemp_declar 
!!  -  use Interface_prop_th_icetemp
!!  -  use Interface_qprod_icetemp
!!  -  use Interface_advec_icetemp_3d
!!  -  use Interface_temp_col
!!  -  use geom_type
!!  -  use temperature_type
!!  -  use ice_flow_type
!!  -  use mask_flgz_type
!!  -  use deformation_type
!!  -  use autre_pr_temp_type
!!  -  use step_temp_type 
!>

Subroutine icetemp(geom_m,temperature_m,Ice_flow_m,Mask_flgz_m,&
     deformation_m,Autre_pr_temp_m,step_temp_m,num_tracebug_m)

  Use Icetemp_declar
  Use Interface_prop_th_icetemp
  Use Interface_qprod_icetemp
  Use Interface_advec_icetemp_3d
  Use Interface_temp_col

  use geom_type
  use temperature_type
  use ice_flow_type
  use mask_flgz_type
  use deformation_type
  use autre_pr_temp_type
  use step_temp_type 

  Implicit None 

  !< Arguments

  type (Geom), intent(in) :: geom_m             
  type (Temperature), intent(inout) :: temperature_m 
  type (Ice_flow), intent(in) :: Ice_flow_m        
  type (Mask_flgz), intent(in) :: Mask_flgz_m   
  type (Deformation), intent(in) :: deformation_m
  type (Autre_pr_temp), intent(inout) :: Autre_pr_temp_m
  type (step_temp), intent(in) :: step_temp_m
  integer,intent(in) :: num_tracebug_m


  Nx_m=size(Temperature_m%Temp_melting,1)
  Ny_m=size(Temperature_m%Temp_melting,2)
  Nz_m=size(Temperature_m%Temp_melting,3)
  NzNzm_m=size(Temperature_m%Temperature,3)
  Nzm_m= NzNzm_m-Nz_m

  ! Allocation de la memoire pour les tableaux dans icetemp_declar

  If (.Not. Allocated(Ct)) then 
     Allocate(Ct(Nx_m,Ny_m,Nz_m))
  end if


  If (.Not. Allocated(Aa)) then 
     ! tab Nn
     Allocate (Aa(Nn),Bb(Nn),Cc(Nn),Rr(Nn),Hh(Nn))  
     ! tab Nznzm_m
     Allocate(Tdot(Nznzm_m))   
     ! tab Nzm+1
     Allocate(Abis(Nzm_m+1),Bbis(Nzm_m+1),Cbis(Nzm_m+1),Rbis(Nzm_m+1),Hbis(Nzm_m+1))                  
     ! tab nz_m
     Allocate(E(Nz_m))        
     !Tab Nx_m,Ny_m
     Allocate(Tbmer(Nx_m,Ny_m),&   
          Alpha(Nx_m,Ny_m),&                   
          Ubar(Nx_m,Ny_m),&                    
          Chalglissx(Nx_m,Ny_m),&
          Chalglissy(Nx_m,Ny_m),&
          Iadvec_w(Nx_m,Ny_m),&
          Iadvec_e(Nx_m,Ny_m),&
          Iadvec_s(Nx_m,Ny_m),&
          Iadvec_n(Nx_m,Ny_m) )      
     ! Tab Nx_m,Ny_m,Nz_m
     Allocate(Chal2_x(Nx_m,Ny_m,Nz_m),&
          Chal2_y(Nx_m,Ny_m,Nz_m),&
          Chal2_z(Nx_m,Ny_m,Nz_m),&
          Chal2_xy(Nx_m,Ny_m,Nz_m),&
          Chaldef_maj(Nx_m,Ny_m,Nz_m),&
          Advecx(Nx_m,Ny_m,Nz_m),&
          Advecy(Nx_m,Ny_m,Nz_m),&
          Advec(Nx_m,Ny_m,Nz_m))
     ! Tab Nx_m,Ny_m,Nz_m,N1poly:N2poly
     Allocate(Chalx(Nx_m,Ny_m,Nz_m,size(Deformation_m%Btt_Deform,4)),&
          Chaly(Nx_m,Ny_m,Nz_m,size(Deformation_m%Btt_Deform,4)))         
     ! Tab Nx_m,Ny_m,N1poly:N2poly
     Allocate(Ffx( Nx_m,Ny_m,size(Deformation_m%Btt_Deform,4)),&
          Ffy( Nx_m,Ny_m,size(Deformation_m%Btt_Deform,4)))                                        
     ! Tab Nx_m,Ny_m,Nznzm_m
     Allocate( T3d_new (Nx_m,Ny_m,Nznzm_m))                  
  end If

  ! init
  Aa=0.    
  Bb=0.              
  Cc=1.             
  Rr=0.              
  Hh=0.              
  Tdot=0  
  !Instruction 
  De=1./(Nz_m-1) 
  Fracq=(1.-(1.-De/2.)**Nfracq)/Nfracq
  Fracq=(1.-(1.-De/2.)**5.)/5.
  E(1)=0.
  E(Nz_m)=1.
  Do K=1,Nz_m
     If ((K.Ne.1).And.(K.Ne.Nz_m)) E(K)=(K-1.)/(Nz_m-1.)
  End Do

  !   Variables Dependant Du Pas De Temps Dtt
  Ctm=step_temp_m%Dtt*Cm/Rom/Cpm/Dzm/Dzm
  Dttdx=step_temp_m%Dtt/step_temp_m%Dx
  Dx11=1./step_temp_m%Dx

  If (Itracebug.Eq.1) Write(Num_tracebug_m,*)' Entree Dans Routine Icetemp Time=',Time

  !________________________________________________________________
  ! La Temperature Du Point De Congelation De L'Eau De Mer
  ! Est Tiree De Jenkins (1991), Tb Est En Degres C
  ! _______________________________________________________________
  Call Temp_mer(Geom_m%Thickness,Mask_flgz_m%Flottant)


  ! Appel Aux Proprietes Thermiques De La Glace : Conductivite, Capacite Claorifique -Temperature_m%Tpmp
  Call Thermal_prop_icetemp(Temperature_m%Temperature,Temperature_m%Temp_melting,Geom_m%Thickness)


  Call Qprod (Iq,geom_m,temperature_m,Ice_flow_m,&
       Mask_flgz_m,deformation_m,Autre_pr_temp_m)


  ! Dans Le Socle : Les elements de La Matrice Tridiag Sont Invariants Dans L'Espace
  Do K=Nz_m+1,Nznzm_m-1
     Aa(K)=-Ctm
     Bb(K)=1.+2.*Ctm
     Cc(K)=-Ctm
  End Do

  ! Conditions Aux Limites Pour Le Socle
  Aa(Nznzm_m)=-1.
  Bb(Nznzm_m)=1.
  Cc(Nznzm_m)=0.

  ! Calcul D'Advection : On Traite Les Tableaux En Une Seule Ligne De Commandes
  ! les tableaux iadvec valent 1 quand la vitesse consideree est upwind et 0 sinon

  !         Advection Selon X
  !         -----------------
  Iadvec_w(:,:)=Nint(Max(Sign(1.0,Ice_flow_m%U_Column_x(:,:)),0.0))
  Iadvec_e(:,:)=Nint(Abs(Min(Sign(1.0,Ice_flow_m%U_Column_x(:,:)),0.0)))
  !         Advection Selon Y
  !         -----------------
  Iadvec_s(:,:)=Nint(Max(Sign(1.0,Ice_flow_m%U_Column_y(:,:)),0.0)) 
  Iadvec_n(:,:)=Nint(Abs(Min(Sign(1.0,Ice_flow_m%U_Column_y(:,:)),0.0)))

  ! Boucle De Resolution Des Temperatures Colonne Par Colonne
  Do J=2,Ny_m-1
     Do I=2,Nx_m-1

        call Advec_icetemp_3d (I,J,Chaldef_maj(I,J,:),Cp(I,J,:),& 
             Advecx(I,J,:),Advecy(I,J,:),Advec(I,J,:),Ct(I,J,:),&
             geom_m,temperature_m,Ice_flow_m,Autre_pr_temp_m,step_temp_m)

        ! variables de calcul dans la glace
        Deh22=2.*De*Geom_m%Thickness(I,J)**2
        Duu=step_temp_m%Dtt/2./De
        Rr(Nznzm_m)=-Dzm*Autre_pr_temp_m%Ghf(I,J)/Cm  ! Dépend De I,J Pour Un Flux Géothermique Variable


        Call Temp_column(I,J,T3d_new,Ct(I,J,:),&
             Mask_flgz_m%Flottant(I,J),Geom_m%Thickness(I,J),Tbmer(I,J),&
             Temperature_m,Autre_pr_temp_m,step_temp_m)

        ! Test Permettant D'Ecrire Les Variables Quand Il Y A Une Erreur
        ! Ifail1 = 1 : Erreur Dans Tridiag
        If (Ifail1.Eq.1) Then
           Write(6,*)'Erreur Dans Tridiag'
           Write(6,*)'I = ',I,' J = ',J
           Write(6,*) 'A, B, C, R, U'
           Do K=1,Nznzm_m
              Write(6,*)'K = ',K
              !Write(6,*) Aa(K),Bb(K),Cc(K),Rr(K),Hh(K)
           End Do
           Stop
        End If
     End Do
  End Do

  Do J=1,Ny_m
     Do I=1,Nx_m
        H1(I,J)=Geom_m%Thickness(I,J)
        B1(I,J)=Geom_m%Base(I,J)
        Temperature_m%Temp_melting(I,J,1)=0.
     End Do
  End Do

  ! Attribution Finale De La Temperature
  ! Limitation À 3deg De Variation Par Dtt Et Pas Plus Froid Que -70

  Do K=1,Nznzm_m
     Do J=1,Ny_m
        Do I=1,Nx_m
           If (Geom_m%Thickness(I,J).Gt.10.) Then

              Tdot(K)=T3d_new(I,J,K)-Temperature_m%Temperature(I,J,K)

              If ((K.Gt.1).And.(Tdot(K).Lt.-3.)) Tdot(K)=-3.
              If ((K.Gt.1).And.(Tdot(K).Gt.3.)) Tdot(K)=3.

              Temperature_m%Temperature(I,J,K)=Temperature_m%Temperature(I,J,K)+Tdot(K)

              If (Temperature_m%Temperature(I,J,K).Lt.-70.) Then
                 Temperature_m%Temperature(I,J,K)=-70.
              Endif
           Endif

        End Do
     End Do
  End Do


  Do K=2,Nz_m
     Do J=1,Ny_m
        Do I=1,Nx_m
           If (Temperature_m%Temperature(I,J,K).Gt.Temperature_m%Temp_melting(I,J,K)) Then
              Temperature_m%Temperature(I,J,K)=Temperature_m%Temp_melting(I,J,K)
              Autre_pr_temp_m%Ibase(I,J)=2
           Endif
        End Do
     End Do
  End Do


  !     Temperature Sur Les Bords : 
  !     -------------------------
  Do K=1,Nz_m
     Do J=1,Ny_m
        !        T(Nx_m,J,K)= T(Nx_m-1,J,K)
        !        T(1,J,K)= T(2,J,K) 
        Temperature_m%Temperature(Nx_m,J,K)= Autre_pr_temp_m%Ts(Nx_m,J)
        Temperature_m%Temperature(1,J,K)= Autre_pr_temp_m%Ts(1,J)
     End Do
     !
     Do I=1,Nx_m
        !        T(I,1,K)=T(I,2,K) 
        !        T(I,Ny_m,K)=T(I,Ny_m-1,K) 
        Temperature_m%Temperature(I,1,K)=Autre_pr_temp_m%Ts(I,1)
        Temperature_m%Temperature(I,Ny_m,K)=Autre_pr_temp_m%Ts(I,Ny_m)
     End Do
  End Do

  Do K=1,Nz_m
     Do J=1,Ny_m
        Do I=1,Nx_m
           If (Geom_m%Thickness(I,J).Le.1.) Temperature_m%Temperature(I,J,K)=Autre_pr_temp_m%Ts(I,J)
        End Do
     End Do
  End Do

  Deallocate (Aa,Bb,Cc,Rr,Hh,Tdot,Abis,Bbis,Cbis,Rbis,Hbis,E,Tbmer,Alpha,Ubar,Chalglissx,Chalglissy,Iadvec_w,Iadvec_e,Iadvec_s,Iadvec_n,&      
       Chal2_x,Chal2_y,Chal2_z,Chal2_xy,Chaldef_maj,Advecx,Advecy,Advec,Chalx,Chaly,Ffx,Ffy,T3d_new)        


  return
End Subroutine Icetemp

!///////////////////////////////////////////////////////////////////////
!///////////////////////////////////////////////////////////////////////
