!> \file ablation_bord.f90
!! Clacul de l'ablation sur le bord
!<
      

!> SUBROUTINE: Ablation_bord
!! Calcule l'ablation pour les points du bord 
!! \author ...
!! \date ....
!! @note cas des bords de calotte : un point du bord peut perdre en plus la glace 
!! amene par l'coulement.
!! @note Calcule l'ablation pour les points du bord avec H=0 mais apport par l'amont
!! diagnostic pour la fermeture du bilan d'eau
!! par convention ablation negative dans la zone d'ablation 
!! @note Used modules:
!! @note  - use module3D_phy
!>

subroutine   ablation_bord


use module3D_phy
implicit none
integer :: ip1,im1,jp1,jm1   ! pour chercher les voisins
integer :: somm


! Dans les zones sans glace ou le calcul icethick a donne un H negatif
! ablbord contient pour l'instant Hneg/dt 
! ailleurs ablbord=0

where (ice(:,:).eq.1)    !  quand glace : abl=bm-acc 
   ablbord(:,:)=0.
endwhere

! pour tre au bord, il suffit que la somme des ice voisins soit > 0
do j=1,ny
   do i=1,nx
      if (ice(i,j).eq.0) then   ! seulment pour les noeuds sans glace
         im1=max(i-1,1)
         jm1=max(j-1,1)
         ip1=min(i+1,nx)
         jp1=min(j+1,ny)

         somm=ice(ip1,j)+ice(im1,j)+ice(i,jp1)+ice(i,jm1)
         if (somm.gt.0) then                               ! voisins englaces
            ablbord(i,j)=bm(i,j)-Acc(i,j)-ablbord(i,j)     !  Ablvrai=bmpdd-Acc-Hcalc/dt 
         else
            ablbord(i,j)=hdot(i,j)-acc(i,j)   ! on ne fond que ce qui est tombe + existant
         endif

      end if
   end do
end do

end subroutine ablation_bord
